import React, {Component} from 'react';
import PropTypes from 'prop-types';


const colorForStatus = new Map([
    ['awaiting', {color: '#BF360C'}],
    ['checking', {color: '#FF6F00'}],
    ['finished', {color: '#43A047'}],
    ['running', {color: '#1976D2'}]
]);

class CompatCard extends Component {
    state = {
        cardHovered: false,
        footerHovered: false
    };

    handleFooterHover = () => {
        this.setState({footerHovered: true});
    };

    handleFooterBlur = () => {
        this.setState({footerHovered: false});
    };

    handleCardHover = () => {
        this.setState({cardHovered: true});
    };

    handleCardBlur = () => {
        this.setState({cardHovered: false});
    };

    handleSelect = () => {
        const {onSelect, id} = this.props;

        onSelect && onSelect(id);
    };

    handleFooterClick = () => {
        const {onFooterButtonClick, id} = this.props;

        onFooterButtonClick && onFooterButtonClick(id);
    };

    handleHeaderClick = (event) => {
        const {onHeaderButtonClick, id} = this.props;

        onHeaderButtonClick && onHeaderButtonClick(id);

        event.stopPropagation();
    };

    renderHeader = () => {
        const {title, subtitle, headerText, headerButton} = this.props;
        const hasHeaderButton = Boolean(headerButton);

        return (
            <div style={style.header}>
                <div style={style.headerContent}>
                    <span style={style.headerTitle}>
                        {title}
                    </span>
                    <span style={style.headerSubtitle}>
                            {subtitle}
                    </span>
                    <span style={style.headerText}>
                            {headerText}
                    </span>
                </div>
                {hasHeaderButton && (
                    <div
                        className="card__header-button"
                        style={style.headerButton}
                        onClick={this.handleHeaderClick}
                    >
                        {headerButton}
                    </div>
                )}
            </div>
        );
    };

    renderContent = () => {
        const {contentTitle, contentText, primaryText, secondaryText} = this.props;

        return (
            <div
                className="card__content-text"
                style={style.cardContentText}
            >
                <span style={style.contentTitle}>
                    {contentTitle}
                </span>
                <span style={style.contentText}>
                    {contentText}
                </span>
                <span style={style.contentPrimary}>
                    {primaryText}
                </span>
                <span style={style.contentSecondary}>
                    {secondaryText}
                </span>
            </div>
        );
    };

    renderFooter = () => {
        const {onFooterButtonClick, footerButtonLabel, footerIcon, footerStatus} = this.props;

        const {status, text} = footerStatus;
        const {color} = colorForStatus.get(status);

        const hasStatusText = Boolean(footerStatus);
        const hasFooterButton = Boolean(onFooterButtonClick);

        if (!hasStatusText && !hasFooterButton) {
            return null;
        }

        return (
            <div
                className="card__footer"
                style={
                    this.state.footerHovered
                        ? {
                            ...style.footer,
                            ...style.footerHover
                        }
                        : style.footer
                }
                onMouseEnter={this.handleFooterHover}
                onMouseLeave={this.handleFooterBlur}
                onClick={this.handleFooterClick}
            >
                {hasStatusText && (
                    <div style={{...style.footerStatus, color}}>
                        {Boolean(footerIcon) && (
                            <span style={style.footerStatusIcon}>
                                {footerIcon}
                            </span>
                        )}
                        <strong style={style.footerStatusName}>
                            {text}
                        </strong>
                    </div>
                )}
                {hasFooterButton && (
                    <div style={style.footerButton}>
                        {footerButtonLabel}
                    </div>
                )}
            </div>
        );
    };

    render() {
        const {image, onSelect} = this.props;

        const hasImage = Boolean(image);
        const canSelect = Boolean(onSelect);

        const boxShadow = this.state.cardHovered
            ? style.cardHover.boxShadow
            : style.card.boxShadow;
        const cursor = canSelect ? 'pointer' : 'default';

        const containerStyle = {
            ...style.cardContainer,
            cursor
        };

        return (
            <div
                style={{...style.card, boxShadow}}
                onMouseEnter={this.handleCardHover}
                onMouseLeave={this.handleCardBlur}
            >
                <div
                    className="card__container"
                    style={containerStyle}
                    onClick={this.handleSelect}
                >
                    {hasImage && (
                        <div style={{
                            ...style.image,
                            backgroundImage: `url(${this.props.image})`
                        }}/>
                    )}
                    <div style={style.cardContent}>
                        {this.renderHeader()}
                        {this.renderContent()}
                    </div>
                </div>
                {this.renderFooter()}
            </div>
        );
    }
}

const truncateStyle = {
    whiteSpace: 'nowrap',
    overflow: 'hidden',
    textOverflow: 'ellipsis'
};

const truncateContainerStyle = {
    minWidth: 0,
    overflow: 'hidden'
};

const style = {
    card: {
        fontFamily: 'Roboto, sans-serif',
        height: 215,
        position: 'relative',
        boxShadow: '0 3px 6px rgba(0, 0, 0, 0.16), 0 3px 6px rgba(0, 0, 0, 0.23)',
        backgroundColor: '#fff'
    },
    cardHover: {
        boxShadow: '0 14px 28px rgba(0, 0, 0, 0.25), 0 10px 10px rgba(0, 0, 0, 0.22)'
    },
    cardContainer: {
        display: 'flex',
        height: '164px'
    },
    cardContent: {
        padding: '12px',
        flex: 1,
        ...truncateContainerStyle
    },
    image: {
        height: 164,
        width: 184,
        backgroundSize: 'cover'
    },
    header: {
        display: 'flex',
    },
    headerContent: {
        flex: 1,
        ...truncateContainerStyle
    },
    headerTitle: {
        display: 'block',
        color: '#263238',
        margin: 0,
        textTransform: 'uppercase',
        fontSize: '.9rem',
        fontWeight: 700,
        ...truncateStyle
    },
    headerSubtitle: {
        display: 'block',
        color: '#90A4AE',
        margin: 0,
        fontSize: '.8rem',
        ...truncateStyle
    },
    headerText: {
        display: 'block',
        color: '#607D8B',
        padding: '3px 0 0',
        fontSize: '.9rem',
        ...truncateStyle
    },
    headerButton: {
        padding: '0 0 12px 12px'
    },
    cardContentText: {},
    contentTitle: {
        display: 'block',
        padding: '5px 0',
        ...truncateStyle
    },
    contentText: {
        display: 'block',
        color: '#607D8B',
        fontSize: '.9rem',
        ...truncateStyle
    },
    contentPrimary: {
        display: 'block',
        padding: '6px 0 0',
        color: '#263238',
        fontSize: '.9rem',
        ...truncateStyle
    },
    contentSecondary: {
        display: 'block',
        padding: '6px 16px 0 0',
        color: '#263238',
        fontSize: '.9rem',
        ...truncateStyle
    },
    footer: {
        backgroundColor: '#E3E2E2',
        display: 'flex',
        height: 51,
        ...truncateContainerStyle
    },
    footerHover: {
        cursor: 'pointer',
        backgroundColor: '#DBDBDB'
    },
    footerStatus: {
        paddingRight: '12px',
        lineHeight: '12px',
        flex: 1,
        display:'flex',
        alignItems: 'center',
        ...truncateStyle
    },
    footerStatusIcon: {
        lineHeight: '12px',
        padding: '0 12px',
    },
    footerStatusName: {
        lineHeight: '51px',
        textTransform: 'uppercase'
    },
    footerButton: {
        lineHeight: '51px',
        borderLeft: '1px solid #CCCBCB',
        padding: '0 12px',
        textTransform: 'uppercase',
        fontSize: '.9rem'
    }
};

CompatCard.propTypes = {
    id: PropTypes.any.isRequired,
    title: PropTypes.string.isRequired,
    image: PropTypes.string,
    subtitle: PropTypes.string,
    headerText: PropTypes.string,
    contentTitle: PropTypes.string,
    contentText: PropTypes.string,
    primaryText: PropTypes.string,
    secondaryText: PropTypes.string,
    headerButton: PropTypes.node,
    footerButtonLabel: PropTypes.string,
    footerIcon: PropTypes.node,
    footerStatus: PropTypes.shape({
        status: PropTypes.string,
        text: PropTypes.string
    }),
    onSelect: PropTypes.func,
    onFooterButtonClick: PropTypes.func,
    onHeaderButtonClick: PropTypes.func
};

export default CompatCard;
